@@ -15,7 +15,7 @@ from py.path import local
 def ignore_encoding_warning():
     with warnings.catch_warnings():
         with contextlib.suppress(NameError):  # new in 3.10
-            warnings.simplefilter("ignore", EncodingWarning)
+            warnings.simplefilter("ignore", EncodingWarning)  # type: ignore [name-defined]  # noqa: F821
         yield
 
 
@@ -1083,14 +1083,14 @@ class TestImport:
         name = "pointsback123"
         ModuleType = type(os)
         p = tmpdir.ensure(name + ".py")
-        for ending in (".pyc", "$py.class", ".pyo"):
-            mod = ModuleType(name)
-            pseudopath = tmpdir.ensure(name + ending)
-            mod.__file__ = str(pseudopath)
-            monkeypatch.setitem(sys.modules, name, mod)
-            newmod = p.pyimport()
-            assert mod == newmod
-        monkeypatch.undo()
+        with monkeypatch.context() as mp:
+            for ending in (".pyc", "$py.class", ".pyo"):
+                mod = ModuleType(name)
+                pseudopath = tmpdir.ensure(name + ending)
+                mod.__file__ = str(pseudopath)
+                mp.setitem(sys.modules, name, mod)
+                newmod = p.pyimport()
+                assert mod == newmod
         mod = ModuleType(name)
         pseudopath = tmpdir.ensure(name + "123.py")
         mod.__file__ = str(pseudopath)
@@ -1522,9 +1522,9 @@ class TestPOSIXLocalPath:
         path1.chown(owner, group)
 
 
-class TestUnicodePy2Py3:
+class TestUnicode:
     def test_join_ensure(self, tmpdir, monkeypatch):
-        if sys.version_info >= (3, 0) and "LANG" not in os.environ:
+        if "LANG" not in os.environ:
             pytest.skip("cannot run test without locale")
         x = local(tmpdir.strpath)
         part = "hällo"
@@ -1532,7 +1532,7 @@ class TestUnicodePy2Py3:
         assert x.join(part) == y
 
     def test_listdir(self, tmpdir):
-        if sys.version_info >= (3, 0) and "LANG" not in os.environ:
+        if "LANG" not in os.environ:
             pytest.skip("cannot run test without locale")
         x = local(tmpdir.strpath)
         part = "hällo"
@@ -1576,4 +1576,4 @@ class TestBinaryAndTextMethods:
         x.write_text(part, "ascii")
         s = x.read_text("ascii")
         assert s == part
-        assert type(s) == type(part)
+        assert type(s) is type(part)
