@@ -103,7 +103,7 @@ def test_strict_option_is_deprecated(pytester: Pytester) -> None:
         def test_foo(): pass
         """
     )
-    result = pytester.runpytest("--strict")
+    result = pytester.runpytest("--strict", "-Wdefault::pytest.PytestRemovedIn8Warning")
     result.stdout.fnmatch_lines(
         [
             "'unknown' not found in `markers` configuration option",
@@ -189,7 +189,7 @@ class TestSkipMsgArgumentDeprecated:
                 pytest.skip(msg="skippedmsg")
             """
         )
-        result = pytester.runpytest(p)
+        result = pytester.runpytest(p, "-Wdefault::pytest.PytestRemovedIn8Warning")
         result.stdout.fnmatch_lines(
             [
                 "*PytestRemovedIn8Warning: pytest.skip(msg=...) is now deprecated, "
@@ -208,7 +208,7 @@ class TestSkipMsgArgumentDeprecated:
                 pytest.fail(msg="failedmsg")
             """
         )
-        result = pytester.runpytest(p)
+        result = pytester.runpytest(p, "-Wdefault::pytest.PytestRemovedIn8Warning")
         result.stdout.fnmatch_lines(
             [
                 "*PytestRemovedIn8Warning: pytest.fail(msg=...) is now deprecated, "
@@ -227,7 +227,7 @@ class TestSkipMsgArgumentDeprecated:
                 pytest.exit(msg="exitmsg")
             """
         )
-        result = pytester.runpytest(p)
+        result = pytester.runpytest(p, "-Wdefault::pytest.PytestRemovedIn8Warning")
         result.stdout.fnmatch_lines(
             [
                 "*PytestRemovedIn8Warning: pytest.exit(msg=...) is now deprecated, "
@@ -245,7 +245,7 @@ def test_deprecation_of_cmdline_preparse(pytester: Pytester) -> None:
 
         """
     )
-    result = pytester.runpytest()
+    result = pytester.runpytest("-Wdefault::pytest.PytestRemovedIn8Warning")
     result.stdout.fnmatch_lines(
         [
             "*PytestRemovedIn8Warning: The pytest_cmdline_preparse hook is deprecated*",
@@ -257,11 +257,17 @@ def test_deprecation_of_cmdline_preparse(pytester: Pytester) -> None:
 def test_node_ctor_fspath_argument_is_deprecated(pytester: Pytester) -> None:
     mod = pytester.getmodulecol("")
 
+    class MyFile(pytest.File):
+        def collect(self):
+            raise NotImplementedError()
+
     with pytest.warns(
         pytest.PytestDeprecationWarning,
-        match=re.escape("The (fspath: py.path.local) argument to File is deprecated."),
+        match=re.escape(
+            "The (fspath: py.path.local) argument to MyFile is deprecated."
+        ),
     ):
-        pytest.File.from_parent(
+        MyFile.from_parent(
             parent=mod.parent,
             fspath=legacy_path("bla"),
         )
@@ -272,7 +278,7 @@ def test_importing_instance_is_deprecated(pytester: Pytester) -> None:
         pytest.PytestDeprecationWarning,
         match=re.escape("The pytest.Instance collector type is deprecated"),
     ):
-        pytest.Instance
+        pytest.Instance  # type:ignore[attr-defined]
 
     with pytest.warns(
         pytest.PytestDeprecationWarning,
@@ -281,6 +287,57 @@ def test_importing_instance_is_deprecated(pytester: Pytester) -> None:
         from _pytest.python import Instance  # noqa: F401
 
 
+def test_fixture_disallow_on_marked_functions():
+    """Test that applying @pytest.fixture to a marked function warns (#3364)."""
+    with pytest.warns(
+        pytest.PytestRemovedIn9Warning,
+        match=r"Marks applied to fixtures have no effect",
+    ) as record:
+
+        @pytest.fixture
+        @pytest.mark.parametrize("example", ["hello"])
+        @pytest.mark.usefixtures("tmp_path")
+        def foo():
+            raise NotImplementedError()
+
+    # it's only possible to get one warning here because you're already prevented
+    # from applying @fixture twice
+    # ValueError("fixture is being applied more than once to the same function")
+    assert len(record) == 1
+
+
+def test_fixture_disallow_marks_on_fixtures():
+    """Test that applying a mark to a fixture warns (#3364)."""
+    with pytest.warns(
+        pytest.PytestRemovedIn9Warning,
+        match=r"Marks applied to fixtures have no effect",
+    ) as record:
+
+        @pytest.mark.parametrize("example", ["hello"])
+        @pytest.mark.usefixtures("tmp_path")
+        @pytest.fixture
+        def foo():
+            raise NotImplementedError()
+
+    assert len(record) == 2  # one for each mark decorator
+
+
+def test_fixture_disallowed_between_marks():
+    """Test that applying a mark to a fixture warns (#3364)."""
+    with pytest.warns(
+        pytest.PytestRemovedIn9Warning,
+        match=r"Marks applied to fixtures have no effect",
+    ) as record:
+
+        @pytest.mark.parametrize("example", ["hello"])
+        @pytest.fixture
+        @pytest.mark.usefixtures("tmp_path")
+        def foo():
+            raise NotImplementedError()
+
+    assert len(record) == 2  # one for each mark decorator
+
+
 @pytest.mark.filterwarnings("default")
 def test_nose_deprecated_with_setup(pytester: Pytester) -> None:
     pytest.importorskip("nose")
@@ -299,7 +356,7 @@ def test_nose_deprecated_with_setup(pytester: Pytester) -> None:
             ...
         """
     )
-    output = pytester.runpytest()
+    output = pytester.runpytest("-Wdefault::pytest.PytestRemovedIn8Warning")
     message = [
         "*PytestRemovedIn8Warning: Support for nose tests is deprecated and will be removed in a future release.",
         "*test_nose_deprecated_with_setup.py::test_omits_warnings is using nose method: `setup_fn_no_op` (setup)",
@@ -327,7 +384,7 @@ def test_nose_deprecated_setup_teardown(pytester: Pytester) -> None:
                 ...
         """
     )
-    output = pytester.runpytest()
+    output = pytester.runpytest("-Wdefault::pytest.PytestRemovedIn8Warning")
     message = [
         "*PytestRemovedIn8Warning: Support for nose tests is deprecated and will be removed in a future release.",
         "*test_nose_deprecated_setup_teardown.py::Test::test is using nose-specific method: `setup(self)`",
