@@ -776,13 +776,13 @@ class TestSorting:
         pytester.makepyfile(
             """\
             class Test1:
-                def test_foo(): pass
-                def test_bar(): pass
+                def test_foo(self): pass
+                def test_bar(self): pass
             class Test2:
-                def test_foo(): pass
+                def test_foo(self): pass
                 test_bar = Test1.test_bar
             class Test3(Test2):
-                def test_baz(): pass
+                def test_baz(self): pass
             """
         )
         result = pytester.runpytest("--collect-only")
@@ -827,11 +827,11 @@ class TestConftestCustomization:
             textwrap.dedent(
                 """\
                 import pytest
-                @pytest.hookimpl(hookwrapper=True)
+                @pytest.hookimpl(wrapper=True)
                 def pytest_pycollect_makemodule():
-                    outcome = yield
-                    mod = outcome.get_result()
+                    mod = yield
                     mod.obj.hello = "world"
+                    return mod
                 """
             ),
             encoding="utf-8",
@@ -855,14 +855,13 @@ class TestConftestCustomization:
             textwrap.dedent(
                 """\
                 import pytest
-                @pytest.hookimpl(hookwrapper=True)
+                @pytest.hookimpl(wrapper=True)
                 def pytest_pycollect_makeitem():
-                    outcome = yield
-                    if outcome.excinfo is None:
-                        result = outcome.get_result()
-                        if result:
-                            for func in result:
-                                func._some123 = "world"
+                    result = yield
+                    if result:
+                        for func in result:
+                            func._some123 = "world"
+                    return result
                 """
             ),
             encoding="utf-8",
@@ -1420,10 +1419,15 @@ def test_package_collection_infinite_recursion(pytester: Pytester) -> None:
 
 
 def test_package_collection_init_given_as_argument(pytester: Pytester) -> None:
-    """Regression test for #3749"""
+    """Regression test for #3749, #8976, #9263, #9313.
+
+    Specifying an __init__.py file directly should collect only the __init__.py
+    Module, not the entire package.
+    """
     p = pytester.copy_example("collect/package_init_given_as_arg")
-    result = pytester.runpytest(p / "pkg" / "__init__.py")
-    result.stdout.fnmatch_lines(["*1 passed*"])
+    items, hookrecorder = pytester.inline_genitems(p / "pkg" / "__init__.py")
+    assert len(items) == 1
+    assert items[0].name == "test_init"
 
 
 def test_package_with_modules(pytester: Pytester) -> None:
@@ -1510,3 +1514,108 @@ def test_package_ordering(pytester: Pytester) -> None:
     # Execute from .
     result = pytester.runpytest("-v", "-s")
     result.assert_outcomes(passed=3)
+
+
+def test_collection_hierarchy(pytester: Pytester) -> None:
+    """A general test checking that a filesystem hierarchy is collected as
+    expected in various scenarios.
+
+    top/
+    ├── aaa
+    │   ├── pkg
+    │   │   ├── __init__.py
+    │   │   └── test_pkg.py
+    │   └── test_aaa.py
+    ├── test_a.py
+    ├── test_b
+    │   ├── __init__.py
+    │   └── test_b.py
+    ├── test_c.py
+    └── zzz
+        ├── dir
+        │   └── test_dir.py
+        ├── __init__.py
+        └── test_zzz.py
+    """
+    pytester.makepyfile(
+        **{
+            "top/aaa/test_aaa.py": "def test_it(): pass",
+            "top/aaa/pkg/__init__.py": "",
+            "top/aaa/pkg/test_pkg.py": "def test_it(): pass",
+            "top/test_a.py": "def test_it(): pass",
+            "top/test_b/__init__.py": "",
+            "top/test_b/test_b.py": "def test_it(): pass",
+            "top/test_c.py": "def test_it(): pass",
+            "top/zzz/__init__.py": "",
+            "top/zzz/test_zzz.py": "def test_it(): pass",
+            "top/zzz/dir/test_dir.py": "def test_it(): pass",
+        }
+    )
+
+    full = [
+        "<Dir test_collection_hierarchy*>",
+        "  <Dir top>",
+        "    <Dir aaa>",
+        "      <Package pkg>",
+        "        <Module test_pkg.py>",
+        "          <Function test_it>",
+        "      <Module test_aaa.py>",
+        "        <Function test_it>",
+        "    <Module test_a.py>",
+        "      <Function test_it>",
+        "    <Package test_b>",
+        "      <Module test_b.py>",
+        "        <Function test_it>",
+        "    <Module test_c.py>",
+        "      <Function test_it>",
+        "    <Package zzz>",
+        "      <Dir dir>",
+        "        <Module test_dir.py>",
+        "          <Function test_it>",
+        "      <Module test_zzz.py>",
+        "        <Function test_it>",
+    ]
+    result = pytester.runpytest("--collect-only")
+    result.stdout.fnmatch_lines(full, consecutive=True)
+    result = pytester.runpytest("top", "--collect-only")
+    result.stdout.fnmatch_lines(full, consecutive=True)
+    result = pytester.runpytest("top", "top", "--collect-only")
+    result.stdout.fnmatch_lines(full, consecutive=True)
+
+    result = pytester.runpytest(
+        "top/aaa", "top/aaa/pkg", "--collect-only", "--keep-duplicates"
+    )
+    result.stdout.fnmatch_lines(
+        [
+            "<Dir test_collection_hierarchy*>",
+            "  <Dir top>",
+            "    <Dir aaa>",
+            "      <Package pkg>",
+            "        <Module test_pkg.py>",
+            "          <Function test_it>",
+            "      <Module test_aaa.py>",
+            "        <Function test_it>",
+            "      <Package pkg>",
+            "        <Module test_pkg.py>",
+            "          <Function test_it>",
+        ],
+        consecutive=True,
+    )
+
+    result = pytester.runpytest(
+        "top/aaa/pkg", "top/aaa", "--collect-only", "--keep-duplicates"
+    )
+    result.stdout.fnmatch_lines(
+        [
+            "<Dir test_collection_hierarchy*>",
+            "  <Dir top>",
+            "    <Dir aaa>",
+            "      <Package pkg>",
+            "        <Module test_pkg.py>",
+            "          <Function test_it>",
+            "          <Function test_it>",
+            "      <Module test_aaa.py>",
+            "        <Function test_it>",
+        ],
+        consecutive=True,
+    )
