@@ -649,7 +649,7 @@ class TestXFail:
             result.stdout.fnmatch_lines(
                 [
                     "*test_strict_xfail*",
-                    "XPASS test_strict_xfail.py::test_foo unsupported feature",
+                    "XPASS test_strict_xfail.py::test_foo - unsupported feature",
                 ]
             )
         assert result.ret == (1 if strict else 0)
@@ -989,33 +989,34 @@ def test_skipped_reasons_functional(pytester: Pytester) -> None:
     pytester.makepyfile(
         test_one="""
             import pytest
-            from conftest import doskip
+            from helpers import doskip
 
-            def setup_function(func):
-                doskip()
+            def setup_function(func):  # LINE 4
+                doskip("setup function")
 
             def test_func():
                 pass
 
-            class TestClass(object):
+            class TestClass:
                 def test_method(self):
-                    doskip()
+                    doskip("test method")
 
-                @pytest.mark.skip("via_decorator")
+                @pytest.mark.skip("via_decorator")  # LINE 14
                 def test_deco(self):
                     assert 0
         """,
-        conftest="""
+        helpers="""
             import pytest, sys
-            def doskip():
+            def doskip(reason):
                 assert sys._getframe().f_lineno == 3
-                pytest.skip('test')
+                pytest.skip(reason)  # LINE 4
         """,
     )
     result = pytester.runpytest("-rs")
     result.stdout.fnmatch_lines_random(
         [
-            "SKIPPED [[]2[]] conftest.py:4: test",
+            "SKIPPED [[]1[]] test_one.py:7: setup function",
+            "SKIPPED [[]1[]] helpers.py:4: test method",
             "SKIPPED [[]1[]] test_one.py:14: via_decorator",
         ]
     )
@@ -1142,12 +1143,10 @@ def test_errors_in_xfail_skip_expressions(pytester: Pytester) -> None:
     """
     )
     result = pytester.runpytest()
-    markline = "                ^"
+    markline = "            ^"
     pypy_version_info = getattr(sys, "pypy_version_info", None)
     if pypy_version_info is not None and pypy_version_info < (6,):
-        markline = markline[5:]
-    elif sys.version_info >= (3, 8) or hasattr(sys, "pypy_version_info"):
-        markline = markline[4:]
+        markline = markline[1:]
 
     if sys.version_info[:2] >= (3, 10):
         expected = [
